function init()
    m.providerURL = getProviderFromSection()
    m.user = getUserFromSection()
    m.global.observeField("deeplink", "deeplinkListener")

    if isUserAuthenticated(m.user, m.providerURL)
        if m.global.deeplink <> invalid
            m.VODDetails = CreateObject("roSGNode", "VODDetailsTask")
            m.VODDetails.providerURL = m.providerURL
            m.VODDetails.email = m.user.email
            m.VODDetails.user = m.user.username
            m.VODDetails.vodName = m.global.deeplink.contentId
            m.VODDetails.control = "RUN"
            m.VODDetails.observeField("VODDetailsDTO", "VODDetailsDTOListener")
        else
            redirectTo("ChannelsScene")
        end if

    else if existingProviderURL(m.providerURL)
        redirectTo("SignInScene")
    else
        redirectTo("ProviderSelectionScene")
    end if

    notifyRokuUserIsLoggedIn()
end function


function notifyRokuUserIsLoggedIn(rsgScreen = invalid as object)
    if type(m.top) = "roSGNode"
        globalNode = m.global
    else
        globalNode = rsgScreen.getGlobalNode()
    end if

    ' get the Roku Analytics Component Library used for RED
    RAC = globalNode.roku_event_dispatcher
    if RAC = invalid then
        RAC = createObject("roSGNode", "Roku_Analytics:AnalyticsNode")
        RAC.debug = true ' for verbose output to BrightScript console, optional
        RAC.init = { RED: {} } ' activate RED as a provider
        globalNode.addFields({ roku_event_dispatcher: RAC })
    end if

    ' dispatch an event to Roku
    RAC.trackEvent = { RED: { eventName: "Roku_Authenticated" } }
end function


function deeplinkListener()
    if isUserAuthenticated(m.user, m.providerURL)
        if m.global.deeplink <> invalid
            m.VODDetails = CreateObject("roSGNode", "VODDetailsTask")
            m.VODDetails.providerURL = m.providerURL
            m.VODDetails.email = m.user.email
            m.VODDetails.user = m.user.username
            m.VODDetails.vodName = m.global.deeplink.contentId
            m.VODDetails.control = "RUN"
            m.VODDetails.observeField("VODDetailsDTO", "VODDetailsDTOLinkDeepListener")
        end if
    end if
end function

function VODDetailsDTOLinkDeepListener()
    if m.VODDetails.VODDetailsDTO = invalid then return invalid
    if m.VODDetails.VODDetailsDTO.status = "NOT_FOUND"
        redirectToWhenDeepLink("ChannelsScene")
        return invalid
    end if

    setVodName(m.global.deeplink.contentId)

    if m.global.deeplink.mediaType = "movie" or m.global.deeplink.mediaType = "episode" or m.global.deeplink.mediaType = "series"
        redirectToWhenDeepLink("VODPlayerScene")
    else if m.global.deeplink.mediaType = "season"
        redirectToWhenDeepLink("VodInternalScene")
    else
        redirectToWhenDeepLink("ChannelsScene")
    end if
end function

function redirectToWhenDeepLink(sceneName)
    scene = CreateObject("roSGNode", sceneName)
    m.top.removeChildIndex(2)
    m.top.appendChild(scene)
    scene.setFocus(true)
end function

function VODDetailsDTOListener()
    if m.VODDetails.VODDetailsDTO = invalid then return invalid
    if m.VODDetails.VODDetailsDTO.status = "NOT_FOUND"
        redirectTo("ChannelsScene")
        return invalid
    end if
    setVodName(m.global.deeplink.contentId)
    if m.global.deeplink.mediaType = "movie" or m.global.deeplink.mediaType = "episode" or m.global.deeplink.mediaType = "series"
        redirectScreenTo("VODPlayerScene")
    else if m.global.deeplink.mediaType = "season"
        redirectScreenTo("VodInternalScene")
    else
        redirectTo("ChannelsScene")
    end if

end function

function redirectTo(scene) as void
    view = CreateObject("roSGNode", scene)
    m.top.appendChild(view)
    view.setFocus(true)
end function

function isUserAuthenticated(user, providerURL) as boolean
    return user <> invalid and providerURL <> invalid
end function

function existingProviderURL(providerURL) as boolean
    return providerURL <> invalid
end function

function setDeepLinking()
    print "deep linking"; m.top.deepLinking
end function

function setCurrentVODOnSection(vod)
    section = CreateObject("roRegistrySection", "Authentication")
    section.Write("currentVOD", vod)
    section.Flush()
end function
